@@ -2,7 +2,7 @@
  controller.js -- The GBrowse controller object
 
  Lincoln Stein <lincoln.stein@gmail.com>
- $Id: controller.js,v 1.69 2008/12/29 17:19:15 lstein Exp $
+ $Id: controller.js,v 1.72 2009/01/06 07:38:24 lstein Exp $
 
 Indentation courtesy of Emacs javascript-mode 
 (http://mihai.bazon.net/projects/emacs-javascript-mode/javascript.el)
@@ -29,6 +29,7 @@ var detail_container_id     = 'detail_panels';
 var external_utility_div_id = 'external_utility_div'; 
 var edit_upload_form_id     = 'edit_upload_form';
 var page_title_id           = 'page_title';
+var galaxy_form_id          = 'galaxy_form';
 var visible_span_id         = 'span';
 var search_form_objects_id  = 'search_form_objects';
 
@@ -99,6 +100,15 @@ var GBrowseController = Class.create({
     }
   }, // end unregister_track
 
+  unregister_gbtrack:
+  function (gbtrack) {
+      var id_hash = this.gbtrackname_to_id.get(gbtrack.track_name);
+      if (id_hash != null)
+           id_hash.unset(gbtrack.track_id);
+      if (this.gbtracks.get(gbtrack.track_id) != null)
+          this.gbtracks.unset(gbtrack.track_id);
+  },
+
   // Pass an iterator to execute something on each track
   // Call as this.each_track(function(track){}) to iterate over all gbtracks.
   // Call as this.each_track('track_name',function(track){}) to iterate over
@@ -262,16 +272,21 @@ var GBrowseController = Class.create({
 
   // Signal Change to Server Methods ********************************
   set_track_visibility:
-  function(track_name,visible) {
+  function(track_id,visible) {
 
-    this.each_track(track_name,function(gbtrack) {
+    var gbtrack    = this.gbtracks.get(track_id);
+    if (gbtrack == null) return;
+
+    var track_name = gbtrack.track_name;
+
+    this.each_track(track_id,function(gbtrack) {
 
       new Ajax.Request('#',{
         method:     'post',
         parameters: {
           set_track_visibility:  1,
-          visible:               visible,
-          track_name:            track_name
+          visible:             visible,
+          track_name:          track_name
         },
         onSuccess: function(transport) {
           if (visible && gbtrack.get_last_update_key() < Controller.last_update_key) {
@@ -358,19 +373,21 @@ var GBrowseController = Class.create({
   }, // end update_coordinates
 
   add_track:
-  function(track_name, onSuccessFunc) {
+  function(track_name, onSuccessFunc, force) {
     var track_names = new Array(track_name);
-    this.add_tracks(track_names, onSuccessFunc);
+    this.add_tracks(track_names,onSuccessFunc,force);
   },
 
   add_tracks:
-  function(track_names, onSuccessFunc) {
+  function(track_names, onSuccessFunc, force) {
+
+    if (force == null) force=false;
 
     var request_str = "add_tracks=1";
     var found_track = false;
     for (var i = 0; i < track_names.length; i++) {
       var track_name = track_names[i];
-      if ( !this.track_exists(track_name) ) {
+      if ( force || !this.track_exists(track_name) ) {
         request_str += "&track_names="+encodeURIComponent(track_name);
         found_track = true;
       }
@@ -391,8 +408,13 @@ var GBrowseController = Class.create({
         var get_tracks = false;
 
         for (var ret_track_id in track_data){
+
+	  if (Controller.gbtracks.get(ret_track_id) != null) {
+	     continue; //oops already know this one
+	  }
+
           var this_track_data = track_data[ret_track_id];
-          var ret_gbtrack = Controller.register_track(
+          var ret_gbtrack     = Controller.register_track(
 	    ret_track_id,
             this_track_data.track_name,
             'standard',
@@ -627,10 +649,10 @@ var GBrowseController = Class.create({
       var loc_str = "?plugin="+plugin_base+";plugin_action="+encodeURI(plugin_action);
       if(source == 'config'){
         var form_element = $("configure_plugin");
-        window.location=loc_str + ";" + form_element.serialize();
+        window.open(loc_str + ";" + form_element.serialize());
       }
       else{
-        window.location=loc_str;
+	window.open(loc_str);
       }
     }
     else if (plugin_type == 'filter'){
@@ -659,7 +681,6 @@ var GBrowseController = Class.create({
 
   commit_file_edit:
   function(edited_file) {
-    $(external_listing_id).innerHTML='<p><b style="background-color:yellow">Working...</b></p>';
     var form_element = $(edit_upload_form_id);
     new Ajax.Request('#',{
       method:     'post',
@@ -670,20 +691,47 @@ var GBrowseController = Class.create({
       onSuccess: function(transport) {
         var results      = transport.responseJSON;
         var file_created = results.file_created;
+	var tracks       = results.tracks;
+
         Controller.wipe_div(external_utility_div_id); 
 
-        if ( 1 == file_created ){
+	var current_tracks = new Hash();
+	Controller.each_track(edited_file,function(gbtrack){
+           current_tracks.set(gbtrack.track_id,gbtrack);
+	});
+
+	var new_tracks = new Hash();
+	tracks.each(function(trackid){
+           new_tracks.set(trackid,1);
+	});
+
+	var tracks_to_delete = current_tracks.keys().findAll(function(trackid) {
+            return new_tracks.get(trackid)==null;
+	});
+	var tracks_to_add    = new_tracks.keys().findAll(function(trackid) {
+	    return current_tracks.get(trackid)==null;
+	});
+	var tracks_to_update = new_tracks.keys().findAll(function(trackid) {
+	    return current_tracks.get(trackid)!= null;
+	});
+
+	tracks_to_update.each(function(id) { 
+	   Controller.rerender_track(id,true);
+	   Controller.update_sections(new Array(external_listing_id));
+	});
+
+	tracks_to_delete.each(function(id) { 
+	   var gbtrack = current_tracks.get(id);
+	   actually_remove(gbtrack.track_div_id);
+	   Controller.unregister_gbtrack(gbtrack);
+	   Controller.update_sections(new Array(track_listing_id,external_listing_id));
+	});
+
+	if (tracks_to_add.length > 0)
           Controller.add_track(edited_file, function(){
             Controller.update_sections(new Array(track_listing_id,external_listing_id));
-          })
-        }
-        else{
-        // update track if it exists
-	 Controller.each_track(edited_file,function(gbtrack) {
-             Controller.rerender_track(gbtrack.track_id,true);
-         });
-         Controller.update_sections(new Array(external_listing_id));
-        }
+        },true);	
+
       } // end onSuccess
     });
   },
@@ -691,6 +739,8 @@ var GBrowseController = Class.create({
   delete_upload_file:
   function(file_name) {
 
+    $(external_listing_id).innerHTML='<p><b style="background-color:yellow">Working...</b></p>';
+
     new Ajax.Request('#',{
       method:     'post',
       parameters: {
@@ -742,7 +792,7 @@ var Controller = new GBrowseController; // singleton
 
 function initialize_page() {
   //event handlers
-  [page_title_id,visible_span_id,search_form_objects_id].each(function(el) {
+    [page_title_id,visible_span_id,galaxy_form_id,search_form_objects_id].each(function(el) {
     if ($(el) != null) {
       Controller.segment_observers.set(el,1);
     }
