@@ -7,7 +7,7 @@ package Bio::Graphics::Browser::GFFPrinter;
 #
 ###################################################################
 
-# $Id: GFFPrinter.pm,v 1.4 2008/12/28 00:05:44 lstein Exp $
+# $Id: GFFPrinter.pm,v 1.7 2009/01/06 07:38:24 lstein Exp $
 
 # Dirt simple GFF3 dumper, suitable for a lightweight replacement to DAS.
 # Call this way:
@@ -29,19 +29,18 @@ sub new {
     my %options = @_;
     my $self    = bless {
         data_source => $options{-data_source},
-        source_name => $options{-source_name},
         segment     => $options{-segment},
         seqid       => $options{-seqid},
         start       => $options{-start},
         segment_end => $options{-end},
         stylesheet  => $options{-stylesheet},
         id          => $options{-id},
-        'dump'      => $options{'-dump'},
+        'dump'      => $options{'-dump'},  # in quotes because "dump" is a perl keyword
         labels      => $options{-labels},
         },
         ref $class || $class;
 
-    $self->check_source();
+    $self->check_source() or return;
     return $self;
 }
 
@@ -52,17 +51,21 @@ sub print_gff3 {
     my $types  = $labels ? $self->labels_to_types($labels) : undef;
     my $files  = $labels ? $self->labels_to_files($labels) : undef;
 
-    $self->print_configuration( $self->data_source(), $labels );
-    $self->print_configuration( $_, [ $_->labels ] ) for @$files;
+    if ($self->get_do_stylesheet) {
+	$self->print_configuration( $self->data_source(), $labels );
+	$self->print_configuration( $_, [ $_->labels ] ) for @$files;
+    }
 
     my %filters
         = map { $_ => $self->data_source->setting( $_ => 'filter' ) || undef }
         @$labels;
 
     my $date = localtime;
+    my $segment = $self->get_segment;
     print "##gff-version 3\n";
     print "##date $date\n";
     print "##source gbrowse gbgff gff3 dumper\n";
+    print "##sequence-region ",$segment->seq_id,':',$segment->start,'..',$segment->end,"\n";
 
     $self->print_gff3_data( $_, $types, \%filters ) for $self->db;
     $self->print_gff3_data($_) for @$files;
@@ -95,7 +98,7 @@ sub segment {
 sub check_source {
     my $self        = shift;
     my $data_source = $self->{data_source};
-    my $source_name = $self->{source_name};
+    my $source_name = $data_source->name();
 
     $source_name =~ s!^/+!!;    # get rid of leading / from path_info()
     $source_name =~ s!/+$!!;    # get rid of trailing / from path_info() !
@@ -104,16 +107,24 @@ sub check_source {
         unless ( $data_source->globals->valid_source($source_name) ) {
             print header('text/plain'), "# Invalid source $source_name; "
                 . "you may not have permission to access this data source.\n";
-            exit 0;
+	    return;
         }
     }
 
-    return;
+    return 1;
 }
 
 sub get_segment {
     my $self = shift;
 
+    # check whether someone called us directly by pasting into location box
+    if ($self->{segment} =~ /^\$segment/) { 
+        print header('text/plain');
+	print "# To share this track, please paste its URL into the \"Enter Remote Annotation\" box\n",
+	"# at the bottom of a GBrowse window and not directly into your browser's Location area.\n";
+	return;
+    }
+
     my ( $seqid, $start, $end )
         = $self->{segment} =~ /^([^:]+)(?::([\d-]+)(?:\.\.|,)([\d-]+))?/;
 
@@ -121,9 +132,9 @@ sub get_segment {
     $start ||= $self->{start} || 1;
     $end   ||= $self->{end};
     unless ( defined $seqid ) {
-        print header('text/plain'),
-            "# Please provide ref, start and end arguments.\n";
-        exit 0;
+        print header('text/plain');
+	print "# Please provide ref, start and end arguments.\n";
+	return;
     }
 
     my $datasource = $self->data_source;
@@ -192,15 +203,14 @@ sub labels_to_types {
     my $data_source = $self->data_source;
 
     # remove dynamic labels, such as uploads
-    my @labels = grep { !/^\w+:/ } @$labels;
-
+    my @labels = grep { /:(overview|region|detail)$/  # keep overview/region sections
+		         || 
+                         !/^\w+:/x                    # discard over dynamic sections
+                       } @$labels;
     my @types;
     for my $l (@labels) {
         my @f = shellwords( $data_source->setting( $l => 'feature' ) || '' );
-        unless (@f) {
-            print "# Unknown track type $l\n";
-            exit 0;
-        }
+	next unless @f;
         push @types, @f;
     }
     return \@types;
@@ -261,9 +271,9 @@ sub print_configuration {
     my @labels = $labels ? @$labels : $config->labels;
 
     for my $l (@labels) {
-	# a special config setting - don't want it to leak through
 
-        next if $l =~ m/^\w+:/;  
+	# a special config setting - don't want it to leak through
+        next if $l =~ m/^\w+:/ && $l !~ m/:(overview|region}detail)$/;  
 	next if $l =~ m/^_scale/;
 
         print "[$l]\n";
